package com.me.pacman.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.me.pacman.Constants;
import com.me.pacman.FontRenderer;
import com.me.pacman.Score;
import com.me.pacman.PacDude;
import com.me.pacman.entity.Direction;

import static com.me.pacman.state.HighScoresState.BUTTON;

public class HighScoreEntryState extends State {

    private Texture background;
    private Score score;

    private final int position;
    private int currLetter;
    private char[] name;

    private boolean flash;
    private float elapsed;

    private boolean buttonPressed;

    private Direction nextDirection;

    public HighScoreEntryState(PacDude game, Score score, int position) {
        super(game);
        this.position = position;
        this.score = score;
        this.currLetter = 0;
        this.flash = false;
        this.elapsed = 0f;
        this.name = score.scorer.toCharArray();
        this.nextDirection = null;
    }

    @Override
    public void setup() {
        this.background = game.assets.getHighScoresBackground();
        Gdx.input.setInputProcessor(this.new Controller());
    }

    @Override
    public void render() {
        game.batch.draw(background, 0, 16);

        game.fontRenderer.setColor(Color.YELLOW);
        game.fontRenderer.draw(game.batch, "high scores", (8 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2, 25 * Constants.TILE_SIZE);

        game.fontRenderer.setColor(Color.CHARTREUSE);
        game.fontRenderer.draw(game.batch, "score  name", (8 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2, 22 * Constants.TILE_SIZE);

        Score[] scores = game.highScores.getHighScores(9);
        for (int n = 0; n < scores.length + 1; n++) {
            int y = (18 - n) * 9;

            if (n == position) {
                game.fontRenderer.setColor(Color.BLUE);
                game.fontRenderer.draw(game.batch, "" + score.score, (8 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2, y);

                for (int i = 0; i < 3; i++) {
                    game.fontRenderer.draw(game.batch, String.valueOf((i == currLetter && flash)? '_' : name[i]), (16 + i) * Constants.TILE_SIZE + Constants.TILE_SIZE/2, y);
                }
            } else {
                Score score = scores[n > position? n - 1 : n];
                game.fontRenderer.setColor(Color.WHITE);
                game.fontRenderer.draw(game.batch, String.format("%-8d%s", score.score, score.scorer), (8 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2, y);
            }
        }

        game.fontRenderer.setColor(buttonPressed ? Color.BLUE : Color.WHITE);
        game.fontRenderer.draw(game.batch, "save", 12 * Constants.TILE_SIZE, 5 * Constants.TILE_SIZE - 1);
    }

    @Override
    public void update(float dt) {
        elapsed += dt;
        if (elapsed >= 1/3f) {
            elapsed = 0f;
            flash = !flash;
        }

        if (nextDirection != null) {
            switch (nextDirection) {
                case UP:
                    int nextIndex = FontRenderer.CHAR_TO_INDEX[name[currLetter]] - 1;
                    if (nextIndex < 0) {
                        nextIndex = 25;
                    }
                    flash = false;
                    name[currLetter] = FontRenderer.CHARS[nextIndex];
                    break;
                case DOWN:
                    nextIndex = FontRenderer.CHAR_TO_INDEX[name[currLetter]] + 1;
                    if (nextIndex > 25) {
                        nextIndex = 0;
                    }
                    flash = false;
                    name[currLetter] = FontRenderer.CHARS[nextIndex];
                    break;
                case LEFT:
                    currLetter--;
                    if (currLetter < 0) {
                        currLetter = 2;
                    }
                    flash = true;
                    break;
                case RIGHT:
                    currLetter++;
                    if (currLetter > 2) {
                        currLetter = 0;
                    }
                    flash = true;
                    break;
            }
            nextDirection = null;
        }
    }

    @Override
    public void dispose() {
        Gdx.input.setInputProcessor(null);
    }

    private void saveScoreAndReturn() {
        score.scorer = String.valueOf(name);
        game.highScores.addScore(score);
        game.setNextState(new HighScoresState(game));
    }

    private final class Controller extends InputAdapter {

        int downX;
        int downY;

        @Override
        public boolean keyDown(int keycode) {
            switch(keycode) {
                case Input.Keys.DOWN:
                    nextDirection = Direction.DOWN;
                    break;
                case Input.Keys.UP:
                    nextDirection = Direction.UP;
                    break;
                case Input.Keys.RIGHT:
                    nextDirection = Direction.RIGHT;
                    break;
                case Input.Keys.LEFT:
                    nextDirection = Direction.LEFT;
                    break;
                case Input.Keys.ENTER:
                case Input.Keys.BACK:
                case Input.Keys.ESCAPE:
                    saveScoreAndReturn();
                    break;
            }
            elapsed = 0f;
            return super.keyDown(keycode);
        }

        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            Vector2 coords = game.viewport.unproject(new Vector2(screenX, screenY));
            if (BUTTON.contains(coords)) {
                buttonPressed = true;
            }
            downX = screenX;
            downY = screenY;
            return true;
        }

        @Override
        public boolean touchUp(int screenX, int screenY, int pointer, int button) {
            if (buttonPressed) {
                Vector2 coords = game.viewport.unproject(new Vector2(screenX, screenY));
                if (BUTTON.contains(coords)) {
                    saveScoreAndReturn();
                }
                return true;
            }
            buttonPressed = false;


            int relaX = screenX - downX;
            int relaY = screenY - downY;

            if (Math.abs(relaX) < 50 && Math.abs(relaY) < 50) {
                // didn't move enough to consider a swipe
                return true;
            }

            if (Math.abs(relaX) > Math.abs(relaY)) {
                // x > y, so moving left-right
                if (relaX > 0) {
                    nextDirection = Direction.RIGHT;
                } else {
                    nextDirection = Direction.LEFT;
                }
            } else {
                // else, moving up/down
                int nextIndex;
                if (relaY > 0) {
                    nextDirection = Direction.DOWN;
                } else {
                    nextDirection = Direction.UP;
                }
            }

            return true;
        }

    }

}
