package com.me.pacman.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.me.pacman.Constants;
import com.me.pacman.PacDude;
import com.me.pacman.SoundManager;

public class MenuState extends LevelState {

    private Texture background;
    private Texture logo;


    public static final int NEW_GAME = 0;
    public static final int HIGH_SCORES = 1;

    private static final BoundingBox NEW_GAME_BOX = new BoundingBox(88, 66, 136, 84);
    private static final BoundingBox HIGH_SCORE_BOX = new BoundingBox(88, 42, 136, 60);

    private int selectedOption;

    public MenuState(PacDude game) {
        super(game);
    }

    @Override
    public void setup() {
        background = game.assets.getMenuBackground();
        logo = game.assets.getLogo();
        logo.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);

        Gdx.input.setInputProcessor(this.new Controller());
        game.sound.play(SoundManager.Effect.BEGINNING);

        switch(Gdx.app.getType()) {
            case Android:
            case iOS:
                selectedOption = -1;
                break;
            default:
                selectedOption = 0;
                break;
        }
    }

    @Override
    public void render() {
        game.batch.draw(background, 0, 16);
        game.batch.draw(logo, 0, 140, 224, 120);


        game.fontRenderer.setColor(selectedOption == 0 ? Color.BLUE : Color.WHITE);
        game.fontRenderer.draw(game.batch, "new", (12 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2, (9 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2);
        game.fontRenderer.draw(game.batch, "game", 12 * Constants.TILE_SIZE, (8 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2);

        game.fontRenderer.setColor(selectedOption == 1 ? Color.BLUE : Color.WHITE);
        game.fontRenderer.draw(game.batch, "high", (12 * Constants.TILE_SIZE), (6 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2);
        game.fontRenderer.draw(game.batch, "scores", (11 * Constants.TILE_SIZE), (5 * Constants.TILE_SIZE) + Constants.TILE_SIZE/2);
    }

    @Override
    public void update(float dt) {}

    private final class Controller extends InputAdapter {

        @Override
        public boolean keyDown(int keycode) {
            switch (keycode) {
                case Input.Keys.UP:
                    selectedOption = selectedOption > 0 ? --selectedOption : 1;
                    game.sound.play(SoundManager.Effect.CHOMP_1);
                    break;
                case Input.Keys.DOWN:
                    selectedOption = selectedOption < 1 ? ++selectedOption : 0;
                    game.sound.play(SoundManager.Effect.CHOMP_2);
                    break;
                case Input.Keys.ENTER:
                    switch (selectedOption) {
                        case NEW_GAME:
                            game.setNextState(new PlayState(game));
                            break;
                        case HIGH_SCORES:
                            game.setNextState(new HighScoresState(game));
                            break;
                    }
                    break;
            }
            return super.keyDown(keycode);
        }

        @Override
        public boolean touchDown(int screenX, int screenY, int pointer, int button) {
            Vector2 coords = game.viewport.unproject(new Vector2(screenX, screenY));
            if (NEW_GAME_BOX.contains(coords)) {
                selectedOption = 0;
            } else if (HIGH_SCORE_BOX.contains(coords)) {
                selectedOption = 1;
            }
            return true;
        }

        @Override
        public boolean touchUp(int screenX, int screenY, int pointer, int button) {
            Vector2 coords = game.viewport.unproject(new Vector2(screenX, screenY));
            if (selectedOption == 0 && NEW_GAME_BOX.contains(coords)) {
                game.setNextState(new PlayState(game));
            } else if (selectedOption == 1 && HIGH_SCORE_BOX.contains(coords)) {
                game.setNextState(new HighScoresState(game));
            } else {
                selectedOption = -1;
            }
            return true;
        }

    }

    @Override
    public void dispose() {
        Gdx.input.setInputProcessor(null);
    }

    public static class BoundingBox {
        private int minX, minY;
        private int maxX, maxY;

        public BoundingBox(int minX, int minY, int maxX, int maxY) {
            this.minX = minX;
            this.minY = minY;
            this.maxX = maxX;
            this.maxY = maxY;
        }

        public boolean contains(int x, int y) {
            return minX < x && x < maxX && minY < y && y < maxY;
        }

        public boolean contains(Vector2 coords) {
            return contains((int) coords.x, (int) coords.y);
        }

    }

}
